package PureMessage::AnnotatedMessage;

use strict;
use base 'PureMessage::Message';

sub minfo;
sub minfo_value;

1;

__END__

=head1 NAME

PureMessage::AnnotatedMessage - RFC822 message with MessageInfo

=head1 DESCRIPTION

This class represents an RFC822 message with attached MessageInfo
(minfo).  The minfo is a header like object that can store meta
information about this message, see L<PureMessage::MessageInfo> for
additional information.  This is an
interface specification.

This class is a subclass of C<PureMessage::Message>.

The following methods are provided in addition to those provided by
C<PureMessage::Message>:

=over

=item $minfo = $msg->minfo

=item $msg->minfo( $minfo )

Get/set the corresponding minfo object.  Setting it updates the file.
It will croak if the update fails.  Update fails unless the mfile
object was created with the C<writable> option.

=item $msg->minfo_value( $key )

Returns the corresponding value.  Shorthand for:

   $msg->minfo->value( $key );

=item $msg->minfo_value( $key, $value,... )

Sets some minfo value.  Shorthand for:

   my $tmp = $msg->minfo;
   $tmp->value($key, $value,... );
   $msg->minfo($tmp);

=back

The following methods are inherited from C<PureMessage::Message>:

=over

=item $msg->header

=item $msg->header_size

=item $msg->message

=item $msg->message_size

=item $msg->message_copy

=item $msg->read

=item $msg->seek

=item $msg->tell

=back

=head1 SEE ALSO

L<PureMessage::Message>, L<PureMessage::MessageInfo>, L<PureMessage::MessageFile>
