package PureMessage::Message;

use strict;

sub header;
sub message;
sub message_size;
sub message_copy;

sub read;
sub seek;
sub tell;

1;

__END__

=head1 NAME

PureMessage::Message - RFC822 message

=head1 DESCRIPTION

This class represents an RFC822 message.  This is an interface specification.

The following methods are provided:

=over

=item $msg->header

Returns the RFC 822 header block as a single multi-lined string.

=item $msg->header( $key )

Read a header value.  In scalar context only the first value of a
repeatable header is returned.

Line breaks within header values are preserved.  If the lines of the
header block are CRLF terminated then the line break in the returned
header value will be the same.  The last line of of multiline header
value will not end with the line terminator.

A single space character is removed from the beginning of a header
value.  Additional leading and all trailing space is preserved.

=item $msg->header_size

Return the size (in bytes) of the RFC 822 header block (including the
terminating blank line).

=item $msg->message

Returns the whole message as a single string.  Note that this will use
up lots of memory if the message is large.  It might be better to read
the message in chunks using the read() method described below.

=item $msg->message_size

Returns the total size (in bytes) of the message.  This includes the
size of the header block.

=item $msg->message_copy( $file )

This method will copy the whole message to the given file.  The $file
argument can be either a file handle or a file name.

The function will croak if not successful.

=item $n = $msg->read( $buf, $size )

Read data from the message.

Parameters and return value same as for perl's built in read function.
The offset parameter is not supported yet.

=item $msg->seek( $pos, $whence )

Move the message read pointer.  Parameters and return value same as
for perl's built in seek function.

=item $msg->tell

Returns the current position.

=back

=head1 SEE ALSO

L<PureMessage::AnnotatedMessage>, L<PureMessage::MessageFile>, L<PureMessage::MessageStore>


