package PureMessage::MessageStore::Dir::Counter;

use strict;
use ActiveState::File::Atomic;
use Carp qw(croak);

sub _cdir {
    my $store = shift;
    $store->dir . "/counters";
}

sub Counters {
    my($class, $store) = @_;
    my $cdir = _cdir($store);
    my @names;
    if (opendir(my $d, $cdir)) {
	@names = map { /^(.+)\.cnt\z/ ? ($1) : () } readdir($d);
	closedir($d);
    }
    return @names;
}

sub new {
    my $class = shift;
    my $store = shift;
    my $name = shift;
    my $cdir = _cdir($store);
    mkdir($cdir, 0755);
    my $filename = "$cdir/$name.cnt";
    my $atomic = ActiveState::File::Atomic->new($filename, @_);
    my $value = $atomic->slurp;
    chomp($value);
    undef($value) unless length($value);
    bless {
	name => $name,
	filename => $filename,
	atomic => $atomic,
	value => $value,
    }, $class;
}

sub value {
    my $self = shift;
    my $old = $self->{value};
    if (@_) {
	croak("Can't set value for closed counter") unless $self->{atomic};

	my $v = shift;
	croak("Counter value set to non-id value '$v'")
	    unless $v =~ /^\d+$/;

	$self->{value} = $v;
	$self->{dirty}++;
    }
    $old;
}

sub inc {
    my $self = shift;
    my $v = $self->value;
    $v++;
    $self->value($v);
}

sub save {
    my $self = shift;
    return 0 unless $self->{dirty};
    my $atomic = delete $self->{atomic};
    return 0 unless $atomic;

    $atomic->commit_string("$self->{value}\n");
    return 1;
}

sub release {
    my $self = shift;
    my $atomic = delete $self->{atomic};
    return 0 unless $atomic;
    
    unlink($self->{filename}) || die "Can't unlink '$self->{filename}': $!";
    $atomic->close;
    1;
}

1;
