package PureMessage::MessageStore::Dir::Index;

use strict;
use ActiveState::Config::Simple qw(config_parse_file);
use Carp qw(croak);


sub new {
    my $class = shift;
    my ($dir, %opts) = @_;

    my $conf_file = delete $opts{config} || "index.conf";
    my $alt_conf = delete $opts{alt_config};
    $opts{verbose} ||= $ENV{PMX_INDEX_DEBUG};
    
    $conf_file = "$dir/$conf_file" unless $conf_file =~ m,^/,;
    my $conf = config_parse_file($conf_file);
    $conf ||= config_parse_file($alt_conf) if defined $alt_conf;
    $conf->{index_type} = 'cdb' unless $conf->{index_type};

    my $idx;
    my $idx_type = "PureMessage::MessageStore::Dir::Index::$conf->{index_type}";
    eval {
	eval "require $idx_type";
	die $@ if $@;
	my %force_verbose = (verbose => 1) if $ENV{PMX_INDEX_DEBUG};
	$idx = $idx_type->new(@_, %force_verbose);
    };
    croak("Failed to load index type $idx_type. $@") if $@;
    return $idx;
}

# Class method
sub get_indexed_fields {
    my $store = shift;
    my $folder = shift;
    my %opts = @_;
    my $dir = $store->dir;
    my $file = "$dir/$folder/index.conf";
    my $alt = "$dir/index.conf";
    my $conf = config_parse_file($file);
    $conf ||= config_parse_file($alt);
    return unless $conf;

    my @fields;
    my $ix = $conf->{index};
    foreach (keys %$ix) {
        my $h = $ix->{$_};
	next if $opts{skip_disabled} && ! _is_idx_enabled($h);
	if (exists $h->{field}) {
	    my $f = $h->{field};
	    push(@fields, ref($f) eq 'ARRAY' ? @$f : $f);
	}
	else {
	    push(@fields, $_);
	}
    }
    return wantarray ? sort(@fields) : @fields;
}

sub _is_idx_enabled {
    my $val = shift;
    return defined $val && lc($val) =~/^yes|on|true|1$/;
}

1;
