package PureMessage::MessageStore::Dir::MessageSet;

use strict;
use Carp qw(croak carp);
use base 'PureMessage::MessageStore::MessageSet';
use PureMessage::MessageStore::MessageSet::QueryParser qw(where_parse extract_ands qp_id_val qp_str_val);

eval {
    require ActiveState::StopWatch;
    ActiveState::StopWatch->import(qw(start_watch read_watch));
};
#warn $@ if $@;

sub _compile_query {
    my $self = shift;
    my $store = $self->{message_store};
    my $folder = $self->{folder} || "cur";
    my $watch = $self->{watch};
    my @extract_fields;
    {
	my %seen;
	for my $fopt (qw(group_by order_by fields)) {
	    next unless $self->{$fopt};
	    (delete $self->{$fopt}), next unless @{$self->{$fopt}};
	    for my $f (map lc, @{$self->{$fopt}}) {
		$f =~ s/\s.*//;
		push(@extract_fields, $f) unless $seen{$f}++;
	    }
	}
    }
    $self->{x_fields} = \@extract_fields;

    my $where = $self->{where};
    $store->log("NOTICE", ($where ? "mset in $folder where $where"
			          : "mset in $folder") );
    my $where_parsed = where_parse($where);
    #require Data::Dump; Data::Dump::dump("WHERE", $where_parsed);
    my @and = extract_ands($where_parsed);
    #require Data::Dump; Data::Dump::dump("AND", \@and);
    
    # Can we manage to limit the range of messages to look at?
    my $min_id = $store->status->{first_id};
    my $max_id = $store->status->{last_id};
    my $idx;
    my $get_idx = sub { $idx ||= $store->_new_index($store->_foldername($folder)) };
    my $idx_ids;
    for my $expr (@and) {
	my $op = $expr->[0];
	my $field = $expr->[1];
	if ($op eq "==" && $field eq "id") {
	    $min_id = $max_id = qp_id_val($expr->[2]);
	}
	elsif (($op eq ">=" || $op eq ">") && $field eq "id") {
	    my $min = qp_id_val($expr->[2]);
	    $min++ if $op eq ">";
	    $min_id = $min if $min_id < $min;
	}
	elsif (($op eq "<=" || $op eq "<") && $field eq "id") {
	    my $max = qp_id_val($expr->[2]);
	    $max-- if $op eq "<";
	    $max_id = $max if !(defined $max_id) || $max_id > $max;
	}
	elsif ($op eq "==" && &$get_idx->indexed($field)) {
	    my $val = qp_str_val($expr->[2], $field);
	    $store->log("DEBUG", "Using index for finding '$val' in field '$field'");
	    push(@{$idx_ids}, &$get_idx->find($field, $val));
	}
    }
    @$idx_ids = reverse @$idx_ids if $idx_ids && $self->{"reverse"};
    # require Data::Dump; Data::Dump::dump("MIN/MAX ID", $min_id, $max_id);
    # require Data::Dump; Data::Dump::dump("IDX", $idx_ids, $idx);
    
    my $match_sub = $self->_generate_match_sub($where_parsed, \@extract_fields);
    #$opts{x_where} = $where_parsed;
    print "#-----\n$match_sub#-----\n" if $self->{verbose};

    # copile match sub
    my $rows = [];  # free lexical for it to see.
    $rows = $self->{rows} if $self->{rows};
    unless ($self->{query_info}) {
	my %query_info = (
	    idx_ids   => $idx_ids,
	    last_id   => $self->{'reverse'} ? $max_id : $min_id,
	    limit     => $self->{limit}, # may be 0 (disabled)
	    min_id    => $min_id,
	    max_id    => $max_id,
	    more_rows => 1,
	    "reverse" => $self->{"reverse"},
	    scan_count => 0,
	);
	$self->{query_info} = \%query_info;
    }

    # make the query info available to the match sub
    my $query_info = $self->{query_info};
    my $stat_scan_msg = 0;
    $self->{rstat_scan_msg} = \$stat_scan_msg;
    $match_sub = eval $match_sub;
    die $@ if $@;
    if ($watch) {
	my $t = read_watch($watch);
	$self->{stat}{compile_time} = $t;
	$store->log("INFO", "mset-compile times: $t");
	$watch = start_watch();
    }
    $self->{query_info}{match_sub} = $match_sub;
    $self->{rows} = $rows;
}

1;
