package PureMessage::MessageStore::Dir::TempFile;

use strict;
use Fcntl qw(O_WRONLY O_CREAT O_EXCL);
use Carp qw(croak);
use PureMessage::_sys ();
my $hostname = PureMessage::_sys::gethostname();

my $count = 0;

sub mk_id {
    my $uniq = shift;
    return join("-", time(), $uniq, $count, $hostname);
}

sub new {
    my($class, $store, $uniq) = @_;
    my $dir = $store->{pmxdir_dir};

    $uniq ||= $$;
    $uniq = sprintf("%010d", $uniq) unless $uniq =~ /[^\d]/;
    $count = ($count + 1) & 0xFFFF;
    $count = sprintf("%03d", $count);

    my $name;

    for (1..4) {
	$name = join("/", $dir, "tmp", mk_id($uniq));
	if (sysopen(my $fd, $name, O_WRONLY | O_CREAT | O_EXCL, 0644)) {
	    return bless {
		pmxdir_fd => $fd,
		pmxdir_fname => $name,
		pmxdir_store => $store,
	    }, $class;
	}
	sleep(1);
    }

    die "Failed to allocate tempfile '$name': $!";
}

sub print {
    my $self = shift;
    if (my $fh = $self->{pmxdir_fd}) {
	return print $fh @_;
    }
    croak("Can't print to closed tempfile");
}

sub flush {
    my $self = shift;
    if (my $fh = $self->{pmxdir_fd}) {
	require IO::Handle;
	return $fh->flush;
    }
    croak("Can't flush closed tempfile");
}

sub close {
    my $self = shift;
    my $status;
    if (my $fh = delete $self->{pmxdir_fd}) {
	$status = close($fh);
	$self->{pmxdir_close_err} = "$!" unless $status;
    }
    return $status;
}

sub filename {
    my $self = shift;
    $self->flush if $self->{pmxdir_fd};
    $self->{pmxdir_fname};
}

sub store {
    my $self = shift;

    if (my $fh = delete $self->{pmxdir_fd}) {
	CORE::close($fh) || die "Can't flush data in '$self->{pmxdir_fname}': $!";
    }
    if (my $err = $self->{pmxdir_close_err}) {
	die "Couldn't flush data in '$self->{pmxdir_fname}': $err";
    }
    if (my $fname = delete $self->{pmxdir_fname}) {
	PureMessage::MessageFile->make($fname, @_);
	my $dir = $self->{pmxdir_store}{pmxdir_dir};
	(my $basename = $fname) =~ s,.*/,,;
	my $newname = "$dir/new/$basename";
	link($fname, $newname)
	    || die "Can't link('$fname', '$newname'): $!";
	unlink($fname) || warn "Can't unlink '$fname': $!";
	return $newname;
    }
    
    croak("Can't store released tempfile");
}

sub release {
    my $self = shift;
    if (my $fname = delete $self->{pmxdir_fname}) {
	unlink($fname) || warn "Can't unlink '$fname': $!";
    }
    if (my $fh = delete $self->{pmxdir_fd}) {
	CORE::close($fh);
    }
    1;
}

sub DESTROY {
    my $self = shift;
    $self->release if $self->{pmxdir_fname};
}

1;
