package PureMessage::MessageStore::Dir::UserView::all;
use strict;
use warnings;
use Carp;
use PureMessage::MessageStore::UserView::Row;

# The 'all' view acts as a base class for the 'visible', 'delete' and 'approve'
# views.

my @view_fields = qw(
    m_enduser
    m_reason
    h_from
    h_to
    m_date
    c_subject_utf8
    m_spam_probability
    );

sub new {
    my $p = shift;
    my $class = ref($p) || $p;

    my %data    = @_;
    my $addr    = delete $data{address};
    my $name    = delete $data{name};
    my $store   = delete $data{store};
    my $reasons = delete $data{reasons};
   
    if ($^W) {
	warn "Invalid option $_ for the visible view."
	    foreach keys %data;
    }
    
    my $self = bless {
	store    => $store,
	name     => $name,
	address  => lc($addr),
	next_row => 0,
    }, $class;
    
    # get our row fetch limit
    $self->{limit} = delete $data{limit} || 0; # 0 is unlimited
    my @_limit = $self->{limit} ? ( limit => $self->{limit} ) : ();
    @_limit = ( limit => 10);
    
    my @where;
    push(@where, $self->_eu_filter()) if $self->_eu_filter();
    push(@where, "any m_mapped_recipients = '$addr'");
    my @reason_query;
    foreach my $reason (@$reasons) {
        push(@reason_query, "(m_reason = '" . lc($reason) .
                        "' or m_reason = '" . ucfirst(lc($reason)) .
                        "' or m_reason = '" . uc($reason) . "')");
    }
    push(@where,'(' . join(' or ',@reason_query) . ')');
    my $where = join(' and ', @where);
    	
    my $mset = $store->message_set(
	fields => \@view_fields,
	where  => join(' and ', @where),
        order_by => ['id desc'],
        @_limit,
    );
    
    $self->{mset} = $mset;
    return $self;
}

sub _eu_filter { undef }

sub name { shift->{name} }

sub fetch {
    my $self = shift;
    # honour the limit setting since mset will turn it off if we have
    # a groupby, but that's ok because it's the actual retrieving of
    # messages that's expensive not the query when fields are indexed
    return undef if $self->{limit} and $self->{next_row} + 1 > $self->{limit};
    my $mset_row = $self->{mset}->fetch($self->{next_row}++);
    return undef unless $mset_row;

    my @field_data = $mset_row->fields;
    return PureMessage::MessageStore::UserView::Row->new(
	fields => \@view_fields,
	id     => $mset_row->message_id,
	data   => \@field_data,
    );
}

1;
