package PureMessage::MessageStore::Fields;

use strict;
use Carp qw(croak);
my %field_code;
my %field_ok = (
    id => 1,
    folder => 1,
    header_size => 1,
    message_size => 1,
);

sub field {
    my($class, $msg, $field) = @_;
    my $code = $field_code{$field} ||= do {
	if (my $code = $class->field_code($field, '$_[0]')) {
	    $code = eval "sub { $code }";
	    die $@ if $@;
	    $code;
	}
    } || croak("Illegal field '$field'");
    return &$code($msg);
}

sub field_code {
    my($class, $field, $msg) = @_;
    return undef unless $class->field_ok($field);
    $msg = '$m' unless defined $msg;
    if ($field =~ s/^([hm])_//) {
	my $method = ($1 eq "h") ? "header" : "minfo_value";
	$field =~ s/_/-/g;
	$field =~ s/([\\\'])/\\$1/g;
	return qq($msg->$method('$field'));
    }
    else {
	return qq($msg->$field);
    }
}

sub field_ok {
    my $class = shift;
    my $field = lc(shift);
    return 0 unless $field =~ /^[a-z]\w+\z/;
    return 1 if $field =~ /^[hm]_[a-z]/;

    if ($field =~ /^c_/) {
	require PureMessage::AnnotatedMessage;
	require PureMessage::Message::ComputedFields;
	return PureMessage::AnnotatedMessage->can($field) ? 1 : 0;
    }

    return 1 if $field_ok{$field};
    return $class->_field_ok($field);
}

# subclasses can override this one
sub _field_ok {
    return 0;
}

1;
