package PureMessage::MessageStore::Message;

use base 'PureMessage::AnnotatedMessage';

sub id;
sub message_store;
sub file_in;
sub trash;

1;

__END__

=head1 NAME

PureMessage::MessageStore::Message - Message stored in a MessageStore

=head1 DESCRIPTION

This class represents an RFC822 message with attached MessageInfo
that is stored in a MessageStore.  This is an
interface specification.

This class is a subclass of C<PureMessage::AnnotatedMessage>.

The following methods are provided in addition to those provided by
C<PureMessage::AnnotatedMessage>:

=over

=item $msg->id

This method returns the primary database key for this message.

=item $msg->message_store

This returns the C<PureMessage::MessageStore> that this message is stored in.

=item $msg->folder

This returns the name of the folder that this message is stored in.

=item $msg->file_in( $folder )

This file file the message in the given folder.  A message can only be
filed in a single folder.

=item $msg->trash

This moves the message to the trash folder.

=item $msg->expire

Deletes this message from the message store.  Meta-information or
statistics about this message might still be left behind.

=back

The following methods are inherited from C<PureMessage::AnnotatedMessage>:

=over

=item $msg->header

=item $msg->header_size

=item $msg->message

=item $msg->message_size

=item $msg->message_copy

=item $msg->read

=item $msg->seek

=item $msg->tell

=item $msg->minfo

=item $msg->minfo_value

=back

=head1 SEE ALSO

L<PureMessage::Message>, L<PureMessage::AnnotatedMessage>
