package PureMessage::MessageStore::MessageSet::Row;
use strict;
use warnings;

# The fields are include each row item except for the last one
sub _field_indices {
    my $self = shift;
    return (0 .. $self->{num_fields} - 1);
}

# The message IDs are tacked onto the end of the row
sub _id_indices {
    my $self = shift;
    return ($self->{num_fields} .. @{$self->{row}} - 1);
}

sub fields {
    my $self = shift;
    return @{$self->{row}}[$self->_field_indices] if wantarray;
    my @i = $self->_field_indices;
    return undef unless @i;
    return $self->{row}[$i[0]];
}

sub message_ids {
    my $self = shift;
    return @{$self->{row}}[$self->_id_indices] if wantarray;
    my @i = $self->_id_indices;
    return scalar(@i);
}

sub message_id {
    my $self = shift;
    return @{$self->{row}}[$self->_id_indices] if wantarray;
    return $self->{row}[($self->_id_indices)[0]];
}

# Set up aliases
sub _ROWS_LOW () { -1; }

for (0 .. _ROWS_LOW) {
    no strict 'refs';
    @{__PACKAGE__ . "_$_" . "::ISA"} = __PACKAGE__;
}

# Return the row array ref as a blessed array ref (or scalar ref if the row 
# only contained a message id).
#
# DEPRECATED in 4.0.4. Prior to this the $mset->{rows} member would have its 
# row type altered from an array ref to a blessed array ref when a row was
# fetch()'ed. This change removes that side effect at the cost of a little more
# per-row overhead. However, most useses of this will release the object 
# reference before fetching another row. This change does not impact the public
# API.
sub _make {
    my($row, $num_fields) = @_;
    my $obj = {row => $row, num_fields => $num_fields};
    return bless $obj, __PACKAGE__ . "_OneID" unless ref($row);

    # num_fields never seems to get below -1 ???
    #return bless $obj, join("_", __PACKAGE__, $num_fields)
    #	if $num_fields <= _ROWS_LOW;
    return bless $obj;
}

package PureMessage::MessageStore::MessageSet::Row_OneID;

sub fields {
    return;
}

sub message_ids {
    my $self = shift;
    return $self->{row} if wantarray;
    return 1;
}

sub message_id {
    my $self = shift;
    return $self->{row};
}

1;
