package PureMessage::MessageStore::ShowTable;

use strict;
use base 'Exporter';
our @EXPORT_OK = qw(show_table);

our $NULL = "NULL";

sub show_table {
    my($set, %opts) = @_;
    my $rows = $set->rows;
    die "Set not executed yet" unless defined $rows;

    # make sure the result set contains all possible rows. 
    my $count = 0;
    $count++ while ($set->fetch($count));
    $rows = $set->rows;
    
    my @out;
    if (my @title = @{$set->fields}) {
	my @w = map length, @title;

	# find optimal width
	my $max = $rows - 1;
	my $i;
	for $i (0 .. $max) {
	    my @field = $set->fetch($i)->fields;
	    my $j = 0;
	    for (@field) {
		$_ = $NULL unless defined;
		$w[$j] = length if $w[$j] < length;
		$j++;
	    }
	}

	_stretch(\@title, \@w);
	my $sep = "+-" . join("-+-", map { "-" x length } @title) . "-+\n";
	push(@out, $sep);
	push(@out, "| ", join(" | ", @title), " |\n");
	push(@out, $sep);

	for $i (0 .. $max) {
	    my @field = $set->fetch($i)->fields;
	    for (@field) { $_ = $NULL unless defined }
	    _stretch(\@field, \@w);
	    push(@out, "| ", join(" | ", @field), " |\n");
	}
	push(@out,  $sep) if $rows;
    }
    push(@out, "  (1 row)\n") if $rows == 1;
    push(@out, "  ($rows rows)\n") if $rows != 1;

    return join("", @out) if defined wantarray;
    print @out;
}

sub _stretch {
    my($text, $w) = @_;
    my $i = 0;
    for (@$text) {
	$_ .= " " x ($w->[$i] - length);
	$i++;
    }
}

1;
